#include "erlang.h"
#include "queuel.h"
#include "qlib.h"

:Evaluate:      LnkBerld::usage = "LnkBerld[x, a] returns the Erlang loss probability for a trunk group x, where x is a real valued variable (double). Second parameter is the Poisson traffic a."
:Begin:
:Function:      Berl_d
:Pattern:       LnkBerld[x_, a_]
:Arguments:     {x,a}
:ArgumentTypes: {Real, Real}
:ReturnType:    Real
:End:

:Evaluate:      LnkBerli::usage = "LnkBerli[n, a] returns the Erlang loss probability for a integer valued trunk group n and Poisson traffic a."
:Begin:
:Function:      Berl_i
:Pattern:       LnkBerli[n_, a_]
:Arguments:     {n,a}
:ArgumentTypes: {Integer, Real}
:ReturnType:    Real
:End:

:Evaluate:      LnkXerl::usage = "LnkXerl[b, a] returns the number of trunks for a loss probability b and Poisson traffic a."
:Begin:
:Function:      Xerl
:Pattern:       LnkXerl[b_, a_]
:Arguments:     {b,a}
:ArgumentTypes: {Real, Real}
:ReturnType:    Real
:End:

:Evaluate:      LnkAerl::usage = "LnkAerl[x, b] returns the Poisson traffic value for a trunk group x and loss probability b."
:Begin:
:Function:      Aerl
:Pattern:       LnkAerl[x_, b_]
:Arguments:     {x,b}
:ArgumentTypes: {Real, Real}
:ReturnType:    Real
:End:

:Evaluate:      LnkQndd1::usage = "LnkQndd1[x, N, D] returns the virtual waiting time distribution for the N*D/D/1 queuing system. The system has N deterministic sources with the same period D, so that the load level of the system is N/D."
:Begin:
:Function:      Qndd1
:Pattern:       LnkQndd1[x_, N_, D_]
:Arguments:     {x, N, D}
:ArgumentTypes: {Real, Integer, Real}
:ReturnType:    Real
:End:

:Evaluate:      LnkQsdd1::usage = "LnkQsdd1[x, D, N] returns the virtual waiting time distribution for the [Sigma]Di/D/1 queuing system. The system has N deterministic sources, each having it's own period D, and a constant service time. Table of periods is given by D."
:Begin:
:Function:      Qsdd1
:Pattern:       LnkQsdd1[x_, D:{__}]
:Arguments:     {x, D}
:ArgumentTypes: {Real, RealList}
:ReturnType:    Real
:End:

:Evaluate:      LnkQmd1::usage = "LnkQmd1[x, rho] returns the virtual waiting time distribution for the M/D/1 queuing system. The system has Poisson arrivals and deterministic service time. Rho is the load level of the system."
:Begin:
:Function:      Qmd1
:Pattern:       LnkQmd1[x_, rho_]
:Arguments:     {x, rho}
:ArgumentTypes: {Real, Real}
:ReturnType:    Real
:End:

:Evaluate:      LnkQmxd1::usage = "LnkQmxd1[x, rho, D] returns the unfinished work tail distribution for the M^x/D/1 queuing system. The system has Poisson batch arrivals and deterministic service time. Parameter x is the amount of unfinished work in the system, rho is the load level of the system and parameter D is the distribution (table) of the batch size probabilities {p1, p2, p3...}. The sum of these probabilities has to be exactly 1."
:Begin:
:Function:      Qmxd1
:Pattern:       LnkQmxd1[x_, rho_, D:{__}]
:Arguments:     {x, rho, D}
:ArgumentTypes: {Real, Real, RealList}
:ReturnType:    Real
:End:

:Evaluate:      LnkMg1::usage = "LnkMg1[x, rho, n] gives the queue length probabilities for the M/G/1 queuing model with load level rho and service time distribution function N ( If N=1: If[x<1,0,1]; If N=2: 1-Exp[-x] )."
:Begin:
:Function:      lnkMg1
:Pattern:       LnkMg1[x_, rho_, D_]
:Arguments:     {x, rho, D}
:ArgumentTypes: {Integer, Real, Manual}
:ReturnType:    Manual
:End:

:Evaluate:      LnkSumMd1::usage = "LnkSumMd1[x, rho] calculates the state probabilities of a M/D/1 queue with load level rho."
:Begin:
:Function:      lnkSumMd1
:Pattern:       LnkSumMd1[x_, rho_]
:Arguments:     {x, rho}
:ArgumentTypes: {Integer, Real}
:ReturnType:    Manual
:End:

:Evaluate:      LnkRecMd1::usage = "LnkRecMd1[x, rho] calculates the state probabilities of a M/D/1 queue with load level rho. This function uses a recursive algorithm."
:Begin:
:Function:      lnkRecMd1
:Pattern:       LnkRecMd1[x_, rho_]
:Arguments:     {x, rho}
:ArgumentTypes: {Integer, Real}
:ReturnType:    Manual
:End:

:Evaluate:      LnkIntFmd1::usage = "LnkIntFmd1[N, rho] is the M/D/1 (virtual) waiting time distribution function (for integral values of x) with load level rho. Service time h is assumed to be 1."
:Begin:
:Function:      IntFmd1
:Pattern:       LnkIntFmd1[N_, rho_]
:Arguments:     {N, rho}
:ArgumentTypes: {Integer, Real}
:ReturnType:    Real
:End:

:Evaluate:      LnkFmd1::usage = "LnkFmd1[x, rho] is the M/D/1 (virtual) waiting time distribution function (it calculates the probability that waiting time W is smaller than x) with load level rho. Service time h is assumed to be 1."
:Begin:
:Function:      Fmd1
:Pattern:       LnkFmd1[x_, rho_]
:Arguments:     {x, rho}
:ArgumentTypes: {Real, Real}
:ReturnType:    Real
:End:

:Evaluate:      LnkMdn::usage = "LnkMdn[x, rho, n] calculates the state probabilities of a M/D/n queue with load level rho and n servers. Accuracy goal is 10^-12."
:Begin:
:Function:      lnkMdn
:Pattern:       LnkMdn[x_, rho_, n_]
:Arguments:     {x, rho, n}
:ArgumentTypes: {Real, Real, Integer}
:ReturnType:    Manual
:End:

:Evaluate:      LnkIntFmdn::usage = "LnkIntFmdn[N, rho, n] is the M/D/n (virtual) waiting time distribution function (for integral values of x) with load level rho. Service time h is assumed to be 1."
:Begin:
:Function:      IntFmdn
:Pattern:       LnkIntFmdn[N_, rho_, n_]
:Arguments:     {N, rho, n}
:ArgumentTypes: {Integer, Real, Integer}
:ReturnType:    Real
:End:

:Evaluate:      LnkFmdn::usage = "LnkFmdn[x, rho, n] is the M/D/n (virtual) waiting time distribution function (it calculates the probability that waiting time W is smaller than x) with load level rho. Service time h is assumed to be 1."
:Begin:
:Function:      Fmdn
:Pattern:       LnkFmdn[x_, rho_, n_]
:Arguments:     {x, rho, n}
:ArgumentTypes: {Real, Real, Integer}
:ReturnType:    Real
:End:

:Evaluate:      LnkFekdn::usage = "LnkFekdn[x, rho, k, n] is the E_k/D/n (virtual) waiting time distribution function (it calculates the probability that waiting time W is smaller than x) with load level rho. Service time h is assumed to be 1."
:Begin:
:Function:      Fekdn
:Pattern:       LnkFekdn[x_, rho_, k_, n_]
:Arguments:     {x, rho, k, n}
:ArgumentTypes: {Real, Real, Integer, Integer}
:ReturnType:    Real
:End:

:Evaluate:      LnkFend1::usage = "LnkFend1[x, n, rho] is the E_n/D/1 (virtual) waiting time distribution function (it calculates the probability that waiting time W is smaller than x) with load level rho. Service time h is assumed to be 1."
:Begin:
:Function:      Fend1
:Pattern:       LnkFend1[x_, n_, rho_]
:Arguments:     {x, n, rho}
:ArgumentTypes: {Real, Integer, Real}
:ReturnType:    Real
:End:

:Evaluate:      LnkBmitra::usage = "LnkBmitra[j, J, C, d, a] returns the blocking probability for a traffic class j (1<=j<=J) from a number of classes J. C is the total capacity of the link. Each traffic class is characterized by it's bandwidth. d is the list of these values and a is the list of traffic intensities for each class."
:Begin:
:Function:      lnkbmitra
:Pattern:       LnkBmitra[j_Integer, J_Integer, C_, d:{__}, a:{__}]
:Arguments:     {j, J, C, d, a}
:ArgumentTypes: {Integer, Integer, Real, IntegerList, RealList}
:ReturnType:    Real
:End:

:Evaluate:      LnkBkaufman::usage = "LnkBkaufman[j, J, C, d, a] returns the blocking probability for a traffic class j (1<=j<=J) from a number of classes J. C is the total capacity of the link. Each traffic class is characterized by it's bandwidth. d is the list of these values and a is the list of traffic intensities for each class. LnkBkaufman is implemented by using the Kaufman and Roberts recursion formula."
:Begin:
:Function:      lnkbkaufman
:Pattern:       LnkBkaufman[k_Integer, K_Integer, C_, dk:{__}, ak:{__}]
:Arguments:     {k, K, C, dk, ak}
:ArgumentTypes: {Integer, Integer, Integer, IntegerList, RealList}
:ReturnType:    Real
:End:
