#include <math.h>
#include "queuel.h"
#include <stdlib.h>

/************************************************************

The Qlib program library has resulted from the work conducted
over several years jointly by

 Laboratory of Telecommunications Technology of
 Helsinki University of Technology (HUT) and

 VTT Information Technology, Telecommunications

The following individuals have contributed to the library:

 Samuli Aalto, Jani Lakkakorpi, Ilkka Norros, Anssi Pirhonen
 and Jorma Virtamo.

The programs can be used and modified freely.

No claims are made about the correctness of the programs
and no liability is taken for any damage caused by the use
of the programs in the library.

Reports of bugs in the program can be sent to qlib@tct.hut.fi

*************************************************************/

double
factorial(int n)
{
  int i = 1;
  double result = 1;
  
  for (i = 1; i <= n; i++)
    result = (result * i);
  return result;
}

double 
*SumMd1(double x, double rho)
{
  int n = 2;
  int v = 1;
  double temp = 0;
  double *P;
  
  P = (double *) malloc((x+1)*sizeof(double));
  if (x >= 0)
    P[0] = (1-rho);
  if (x >= 1)
    P[1] = ( (1-rho) * (exp(rho) - 1) );
  if (x > 1) {
    for (n = 2; n <= x; n++) {
      temp = 0;
      for (v = 1; v < n; v++) {
        temp = temp + (  pow((-1),(n-v)) * exp(v*rho) * (  pow((v*rho),(n-v))/factorial(n-v) +
							   pow((v*rho),(n-v-1))/factorial(n-v-1)  )  );
      }
      temp = temp + exp(n*rho);
      P[n] = (1-rho)*temp;
    }
    return P;
  }
  else
    return NULL;
}

double
*RecMd1(int x, double rho)
{
  int i = 1;
  int j = 1;
  double temp = 0;
  double *P;
  
  P = (double *) malloc((x+1)*sizeof(double));
  P[0] = 1-rho;
  if (x>0)
    P[1] = (1-rho)*(exp(rho)-1);
  if (x>1)
    P[2] = (1-rho)*(-exp(rho)*(1+rho)+exp(2*rho) );
  if (x>2) {
    for (i = 2; i < x; i++) {
      temp = 0;
      for(j = 2; j <= i; j++) {
        temp = temp + (P[j] * pow(rho,(i-j+1)) / factorial(i-j+1) * exp(-rho) );
      }
      P[i+1] = 1/(exp(-rho)) * ( P[i] - (P[0]+P[1]) * pow(rho,i) / factorial(i) * exp(-rho) - temp );
    }
  }
  return P;
}

double      
IntFmd1(int N, double rho)
{
  int n = 0;
  double sum = 0;
  double *probabilities;
  
  probabilities = RecMd1(N, rho);
  for (n = 0; n <= N; n++)
    sum = sum + probabilities[n];
  return sum;  
}

double						  
Fmd1(double x, double rho)
{
  int n = 0;
  int T = floor(x);
  double t = 0;
  double sum = 0;
  
  t=x-T;
  if(t == 0)
    return IntFmd1(T, rho);
  if(t > 0) {
    for (n = 0; n <= T; n++) {
      sum = sum + pow(-rho*t,n)/factorial(n) *  IntFmd1(T-n, rho);
    }
    return exp(rho*t)*sum;
  }
  else 
    return 0;	
}








