#include <math.h>
#include <stdlib.h>
#include "erlang.h"

/************************************************************

The Qlib program library has resulted from the work conducted
over several years jointly by

 Laboratory of Telecommunications Technology of
 Helsinki University of Technology (HUT) and

 VTT Information Technology, Telecommunications

The following individuals have contributed to the library:

 Samuli Aalto, Jani Lakkakorpi, Ilkka Norros, Anssi Pirhonen
 and Jorma Virtamo.

The programs can be used and modified freely.

No claims are made about the correctness of the programs
and no liability is taken for any damage caused by the use
of the programs in the library.

Reports of bugs in the program can be sent to qlib@tct.hut.fi

*************************************************************/

/* Blocking in multiple bit speed system
 * Kaufman & Roberts recursion formula
 * j    - traffic class
 * J    - total number of classes
 * C    - total number of channels
 * dk[] - channels required by each class
 * ak[] - traffic intensity of each class 
 */

static double get_Q(int, int, int *, double *, double *, int);

double
Bkaufman(int j, int J, int C, int *dk, double *ak)
{
  double sum1 = 0, sum2 = 1;
  double *frame;
  int d,larg;
  
  /*  Alloc hash table for previous values of Q(d). Size = max(dk) */
  
  larg = dk[vector_max(dk, J)];
  frame = calloc(larg, sizeof(double));
  if (frame == NULL) exit(1);
  frame[0] = 1;
  
  /* Calculate sum in two parts */
  
  for (d = 1; d < C - dk[j-1] + 1; d++)
    sum2 += get_Q(d, J, dk, ak, frame, larg);
  
  for (d = C - dk[j-1] + 1; d <= C; d++)
    sum1 += get_Q(d, J, dk, ak, frame, larg);
  
  free(frame);
  return sum1/(sum1+sum2);
}

static double
get_Q(int d, int J, int *dk, double *ak, double *frame, int winsize)
{
  int i, offset;
  double sum = 0, qprev;
  
  if (d == 0) return 1;
  if (d < 0) return 0;
  
  /*  Hash function for values of Q is Q(d) = d % winsize    */
  
  for (i = 0; i < J; i++) {
    if (d - dk[i] < 0) 
      qprev = 0;
    else {
      offset = (d - dk[i]) % winsize; 
      qprev = frame[offset];     
    }
    sum += ak[i] * dk[i] * qprev;
  }
  sum /= d;
  frame[d % winsize] = sum;    /* Save this value in table */
  
  return sum;
}
