#include <math.h>
#include "queuel.h"
#include <stdlib.h>

/************************************************************
  
The Qlib program library has resulted from the work conducted
over several years jointly by

 Laboratory of Telecommunications Technology of
 Helsinki University of Technology (HUT) and

 VTT Information Technology, Telecommunications

The following individuals have contributed to the library:

 Samuli Aalto, Jani Lakkakorpi, Ilkka Norros, Anssi Pirhonen
 and Jorma Virtamo.

The programs can be used and modified freely.

No claims are made about the correctness of the programs
and no liability is taken for any damage caused by the use
of the programs in the library.

Reports of bugs in the program can be sent to qlib@tct.hut.fi

*************************************************************/

double
Sintegrate( double (*)(double), int, int, double, double, double, double);

double
*Mg1(int x, double rho, double (*D)(double)) { 
  double *o;
  double *a;
  double *p;
  double lambda = rho;
  int i, j, k, n, nfactorial = 0;
  double cum = 0;
  
  a = (double *) malloc((x+1) * sizeof(double));
  p = (double *) malloc((x+1) * sizeof(double));
  
  for (n = 0; n <= x; n++) {
    i = n;
    nfactorial = 1;
    while (i > 1) {
      nfactorial = (nfactorial * i);
      i = (i-1); 
    };
    a[n] = Sintegrate(D, n, nfactorial, lambda, 0.0, 30.0, 100000);
  }
  
  p[0] = (1 - rho);
  
  for (j = 1; j <= x; j++) {
    cum = 0;
    for (k = 1; k <= (j-1); k++) {
      cum = (cum + (a[j-k] * p[k]));
    }
    p[j] = ( 1/(1-lambda*a[0]) * (lambda*a[j-1]*p[0] + lambda*cum) );
  }  
  
  o = (double *) malloc((x+1)*sizeof(double));
  for (i = 0; i <= x; i++) {
    o[i]=p[i];
  };
  
  free(a);
  free(p);
  
  return o;
}

double
Sintegrate(double (*fp) (double), int n, int nfactorial, double lambda,
	   double a, double b, double k) {
  /* fp is a pointer to the function to be integrated; a and b are the limits
     of integration; k is the number of integration intervals. Numerical
     integration is done by using Simpson's 1/3 rule. */
  
  double result, fa, fb, odds=0, evens=0, h;
  int i;
  h = (b-a)/k; /* Subinterval length */
  
  fa = exp(-lambda*a)*pow(lambda*a,n)/nfactorial*(1-(*fp)(a));
  fb = exp(-lambda*b)*pow(lambda*b,n)/nfactorial*(1-(*fp)(b));
  
  for (i = 0; i <= (k/2.-1); i++) {
    odds += exp(-lambda*(a+(2*i+1)*h))*pow(lambda*(a+(2*i+1)*h),n)/nfactorial*(1-(*fp)(a+(2*i+1)*h));
  }
  
  for (i = 1; i <= (k/2.-1); i++) {
    evens += exp(-lambda*(a+2*i*h))*pow(lambda*(a+2*i*h),n)/nfactorial*(1-(*fp)(a+2*i*h));
  }
  
  result = h*(fa + fb + 4*odds + 2*evens)/3.;
  
  return result;
}   
