#include <math.h>
#include <stdlib.h>
#include "erlang.h"

/************************************************************

The Qlib program library has resulted from the work conducted
over several years jointly by

 Laboratory of Telecommunications Technology of
 Helsinki University of Technology (HUT) and

 VTT Information Technology, Telecommunications

The following individuals have contributed to the library:

 Samuli Aalto, Jani Lakkakorpi, Ilkka Norros, Anssi Pirhonen
 and Jorma Virtamo.

The programs can be used and modified freely.

No claims are made about the correctness of the programs
and no liability is taken for any damage caused by the use
of the programs in the library.

Reports of bugs in the program can be sent to qlib@tct.hut.fi

*************************************************************/

static double newton(double, int *, double *, int);
static double func(double, int *, double *, int);
static double func_d(double, int *, double *, int);

/* Poisson limit of blocking in multiple bit speed system */
/*                                                        */
/* j    - traffic class                                   */
/* J    - total number of classes                         */
/* C    - total number of channels                        */ 
/* dk[] - channels required by each class                 */ 
/* ak[] - traffic intensity of each class                 */
/* Formula in Mitra's paper                               */

double
Bmitra(int j, int J, double C, int *d, double *aval)
{
  double z, fz = 0, vz = 0, powzd, M, L, *a;
  int i, lind = vector_max(d, J);
  double iter_init;
  
  a = calloc(J, sizeof(double));
  if (a == NULL) exit(1);
  
  for (i=0; i < J; i++) 
    a[i] = aval[i] / C;
  
  iter_init = pow(1/((double)d[lind]*a[lind]), 1/(double)d[lind]);
  
  z = newton(iter_init, d, a, J);
  
  for (i = 0; i < J; i++) {
    powzd = pow(z, d[i]);
    fz += a[i] * (powzd - 1);
    vz += sqr(d[i]) * a[i] * powzd;
  }
  fz -= log(z);
  
  M = sgn(1-z) * sqrt(-C * fz);
  M = 0.5 * erfc(M);
  
  if (M == 0) 
    L = (1 - pow(z, d[j-1])) * sqrt(-2*fz) / (sqrt(-2 * fz) - sgn(1-z) * 
					      sqrt(vz) * (1 - z)); 
  else { 
    M += exp(C*fz)/sqrt(2*M_PI*C)*(1/(sqrt(vz)*(1-z)) - sgn(1-z)/sqrt(-2*fz));
    L = exp(C*fz)*(1-pow(z, d[j-1]))/ (sqrt(2*M_PI*C*vz)*(1-z)*M);
  }
  
  free(a);
  return L;
  
}

/* Newton method for finding z in func(z) = 0 */

static double
newton(double z, int *d, double *a, int J)
{
  double y, zprev;
  
  do
    {
      y = func(z, d, a, J);
      zprev = z;
      z = z - y / func_d(z, d, a, J);
    }
  while (fabs (z / zprev - 1) > 1e-15);
  
  
  
  return z;
}


/* func(z) = Sum[d[j] a[j] z ^ d[j] , {j, 0, J-1}] - 1 */

static double
func(double z, int *d, double *a, int J)
{
  int j;
  double sum = 0;
  
  for (j = 0; j < J; j++)
    sum += d[j] * a[j] * pow(z, d[j]);
  
  return sum - 1;
  
}

/* func'(z) */

double
func_d(double z, int *d, double *a, int J)
{
  int j;
  double sum = 0;
  
  for(j = 0; j < J; j++)
    sum += sqr(d[j]) * a[j] * pow(z, d[j] - 1);
  
  return sum;
  
}
